package com.ninegag.android.blitz2

import androidx.annotation.CallSuper
import io.reactivex.Scheduler
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import java.util.ArrayList

open class RxBlitz<Model, PagingKey, QueryParam: DataSource.QueryParam>(
        private val dataSource: RxDataSource<Model, PagingKey, QueryParam>,
        private val fetchScheduler: Scheduler = Schedulers.io(),
        private val mainScheduler: Scheduler = AndroidSchedulers.mainThread()
) : Blitz<Model>() {

    private val pageInfos: MutableList<PagingInfo<Model, PagingKey>> = ArrayList()
    private val disposables = CompositeDisposable()

    val listState = dataSource.listStateRelay
    val errorState = dataSource.errorStateRelay

    fun registerSubscriptions() {

    }

    fun unregisterSubscriptions() {
        disposables.dispose()
        listeners?.clear()
    }

    override fun clear() {
        super.clear()
        pageInfos.clear()
    }

    override fun initLoad() {
        Timber.d("initLoad: ")

        dataSource.listStateRelay.accept(BlitzState.STATE_INIT_LOADING)
        val query = dataSource.createQueryParam(null)

        listeners?.let {
            for (i in listeners.size - 1 downTo 0) {
                listeners[i].onInit()
            }
        }

        disposables.add(dataSource.createInitLoadObservable(query)
                .subscribeOn(fetchScheduler)
                .observeOn(mainScheduler)
                .subscribe({ blitzQueryResult ->
                    clear()
                    addAll(blitzQueryResult.items)
                    pageInfos.add(PagingInfo(blitzQueryResult))

                    if (size > 0) {
                        if (dataSource.hasNext() && dataSource.hasPrev()) {
                            listState.accept(BlitzState.STATE_FINISH_LOADING_HAS_PREV_AND_NEXT)
                        } else if (dataSource.hasNext()){
                            listState.accept(BlitzState.STATE_FINISH_LOADING_HAS_NEXT)
                        } else if (dataSource.hasPrev()) {
                            listState.accept(BlitzState.STATE_FINISH_LOADING_HAS_PREV)
                        } else {
                            listState.accept(BlitzState.STATE_FINISH_LOADING_NEXT_END_OF_LIST)
                        }
                    } else {
                        listState.accept(BlitzState.STATE_INIT_LOADING_EMPTY_LIST)
                    }

                    listeners?.let {
                        for (i in listeners.size - 1 downTo 0) {
                            listeners[i].onInitDone(blitzQueryResult.items, hasNext())
                        }
                    }

                }, { e ->
                    errorState.accept(e)
                    listState.accept(BlitzState.STATE_FINISH_LOADING_ERROR)
                }))

    }

    @SuppressWarnings("unchecked")
    @Override
    @CallSuper
    override fun remoteRefresh(param: BlitzQueryParam?) {
        super.remoteRefresh(param)
    }

    override fun remoteRefresh() {
        val query = dataSource.createInvalidationQueryParam()
        dataSource.listStateRelay.accept(BlitzState.STATE_REFRESHING)

        listeners?.let {
            for (i in listeners.size - 1 downTo 0) {
                listeners[i].onStartRefresh()
            }
        }

        disposables.add(dataSource.createInitLoadObservable(query)
                .lastElement()
                .subscribeOn(fetchScheduler)
                .observeOn(mainScheduler)
                .subscribe({ blitzQueryResult ->
                    clear()
                    addAll(blitzQueryResult.items)
                    pageInfos.add(PagingInfo(blitzQueryResult))

                    if (size > 0) {
                        if (dataSource.hasNext() && dataSource.hasPrev()) {
                            listState.accept(BlitzState.STATE_FINISH_LOADING_HAS_PREV_AND_NEXT)
                        } else if (dataSource.hasNext()){
                            listState.accept(BlitzState.STATE_FINISH_LOADING_HAS_NEXT)
                        } else if (dataSource.hasPrev()) {
                            listState.accept(BlitzState.STATE_FINISH_LOADING_HAS_PREV)
                        } else {
                            listState.accept(BlitzState.STATE_FINISH_LOADING_NEXT_END_OF_LIST)
                        }
                    } else {
                        listState.accept(BlitzState.STATE_INIT_LOADING_EMPTY_LIST)
                    }

                    listeners?.let {
                        for (i in listeners.size - 1 downTo 0) {
                            listeners[i].onRefreshDone(blitzQueryResult.items, hasNext())
                        }
                    }

                }, { e ->
                    errorState.accept(e)
                    listState.accept(BlitzState.STATE_FINISH_LOADING_ERROR)
                }))
    }

    override fun loadNext() {
        listState.accept(BlitzState.STATE_LOADING_MORE)
        val query = dataSource.createQueryParam(pageInfos[pageInfos.size - 1])

        disposables.add(dataSource.createLoadNextObservable(query)
                .lastElement()
                .subscribeOn(fetchScheduler)
                .observeOn(mainScheduler)
                .subscribe({ blitzQueryResult ->
                    addAll(blitzQueryResult.items)
                    pageInfos.add(PagingInfo(blitzQueryResult))

                    if (dataSource.hasNext()) {
                        listState.accept(BlitzState.STATE_FINISH_LOADING_HAS_NEXT)
                    } else {
                        listState.accept(BlitzState.STATE_FINISH_LOADING_NEXT_END_OF_LIST)
                    }

                    listeners?.let {
                        for (i in listeners.size - 1 downTo 0) {
                            listeners[i].onLoadNextDone(blitzQueryResult.items, hasNext())
                        }
                    }

                }, { e ->
                    errorState.accept(e)
                }))
    }

    override fun loadPrev() {
        dataSource.listStateRelay.accept(BlitzState.STATE_LOADING_MORE)
        val query = dataSource.createQueryParam(if (pageInfos.size >= 0) pageInfos[0] else null)

        disposables.add(dataSource.createLoadPrevObservable(query)
                .lastElement()
                .subscribeOn(fetchScheduler)
                .observeOn(mainScheduler)
                .subscribe({ blitzQueryResult ->
                    // Prepend items
                    addAll(0, blitzQueryResult.items)
                    pageInfos.add(0, PagingInfo(blitzQueryResult))

                    if (dataSource.hasPrev()) {
                        listState.accept(BlitzState.STATE_FINISH_LOADING_HAS_PREV)
                    } else {
                        listState.accept(BlitzState.STATE_FINISH_LOADING_PREV_END_OF_LIST)
                    }

                    listeners?.let {
                        for (i in listeners.size - 1 downTo 0) {
                            listeners[i].onLoadPrevDone(blitzQueryResult.items)
                        }
                    }
                }, { e ->
                    errorState.accept(e)
                }))
    }

    override fun silentRemoteRefresh(param: BlitzQueryParam) {
        // TODO: 3/4/2019 Implement later, this is for background updating the list, without affecting UI
    }

    override fun hasNext(): Boolean {
        return dataSource.hasNext()
    }

    override fun hasPrev(): Boolean {
        return dataSource.hasPrev()
    }

    override fun onAddItemFromAdapter(position: Int, item: Model) {

    }

    override fun onAddItemFromAdapter(item: Model) {

    }

    override fun onRemoveItemFromAdapter(position: Int) {

    }
}