package com.under9.android.comments.data.repository

import android.util.Log
import com.under9.android.comments.controller.DataController
import com.under9.android.comments.data.processor.BaseApiResponseProcessor
import com.under9.android.comments.data.query.comment.CommentListQueryParam
import com.under9.android.comments.data.query.comment.CommentListQueryResult
import com.under9.android.comments.helper.CommentHelper
import com.under9.android.comments.helper.CommentWriterV2
import com.under9.android.comments.model.Comment
import com.under9.android.comments.model.api.ApiComment
import com.under9.android.comments.model.api.ApiCommentList
import io.reactivex.Flowable
import timber.log.Timber

class CommentListProcessor(dataController: DataController,
                           private val commentListQueryParam: CommentListQueryParam,
                           private val commentWriter: CommentWriterV2): BaseApiResponseProcessor<ApiCommentList>(dataController) {

    override fun process(apiResponse: ApiCommentList): Flowable<CommentListQueryResult> {

        var hasNext = false

        dataController.runInTx{
            var parentComment: Comment? = null
            var startLeft = 1
            if (commentListQueryParam.isRefresh) {
                // If it's a sublevel refresh, we clean up sublevel's comment only, not the whole list
                if (commentListQueryParam.level > 0) {
                    val commentListItem = dataController.getCommentListItemByCommentId(commentListQueryParam.parentCommentId, commentListQueryParam.url, commentListQueryParam.order)
                    commentListItem.comment?.childrenComments?.forEachIndexed { index, child ->
                        if (index > 0) {
                            dataController.deleteComment(child)
                            Timber.d("deleting comment=$child")
                        }
                    }
                    val lastItem = dataController.getCommentListItemByCommentId(commentListQueryParam.parentCommentId, commentListQueryParam.url, commentListQueryParam.order)
                    if (lastItem != null) {
                        lastItem.refresh()
                        // insert after the last first-level comment
                        startLeft = lastItem.rgt!! + 1
                    }
                    commentListItem.comment?.resetChildrenComments()
                    parentComment = commentListItem.comment
                } else {
                    val commentListItems = dataController.getCommentListItemByUrl(commentListQueryParam.url, commentListQueryParam.order)
                    commentListItems.forEach {
                        it.comment?.childrenComments?.forEachIndexed { index, child ->
                            if (index > 0) {
                                dataController.deleteComment(child)
                            }
                        }
                    }

                    dataController.clearCommentList(commentListQueryParam.url, commentListQueryParam.order)
                    startLeft = 1
                }

                dataController.clearAllCache()
            } else {
                if (commentListQueryParam.parentCommentId == null) {
                    /* First Level comment, find the last item */

                    //since we do not have a root node, we need to use the "right" of the last node
                    val lastItem = dataController.getLastTopLevelCommentListItem(commentListQueryParam.url, commentListQueryParam.order)
                    if (lastItem != null) {
                        lastItem.refresh()
                        // insert after the last first-level comment
                        startLeft = lastItem.rgt!! + 1
                    }
                } else {
                    val item = dataController.getCommentListItemByCommentId(commentListQueryParam.parentCommentId, commentListQueryParam.url, commentListQueryParam.order)
                    if (item != null) {
                        item.refresh()
                        startLeft = item.rgt
                        parentComment = item.comment
                    }
                }
            }



            apiResponse.payload.run {
                hasNext = this.hasNext

                if (parentComment != null) {
                    parentComment.hasNext = hasNext
                    dataController.updateComment(parentComment)
                }

                CommentHelper.processApiCommentList(comments, apiResponse, null)
                val validatedComments = validateComments(comments)

                dataController.replaceCommentMeta(commentListQueryParam.url)
                commentWriter.appendApiCommentsToDB(commentListQueryParam.url, commentListQueryParam.order, validatedComments.toArray(arrayOfNulls(validatedComments.size)), parentComment, startLeft, commentListQueryParam.shouldOverrideLikeStatus)
            }
        }

        return Flowable.just(CommentListQueryResult(hasNext, dataController.getLastOrderKey(commentListQueryParam.url, commentListQueryParam.order)))
    }

    private fun validateComments(comments: Array<ApiComment>): ArrayList<ApiComment> {
        val validatedComments = ArrayList<ApiComment>()
        for (i in comments.indices) {
            comments[i].run {
                validatedComments.add(this)
                Log.d("CLProcessor", "this.parent=${this.parent}, text=${this.text}")
            }
        }
        return validatedComments
    }
}