package com.under9.android.comments.data.repository

import androidx.annotation.VisibleForTesting
import androidx.collection.ArrayMap
import android.util.Log
import com.google.gson.Gson
import com.under9.android.comments.AppOptions
import com.under9.android.comments.CommentSystem
import com.under9.android.comments.api.CommentApiService
import com.under9.android.comments.controller.DataController
import com.under9.android.comments.data.processor.UserStatusProcessor
import com.under9.android.comments.data.query.user.UserStatusQueryResult
import com.under9.android.lib.network.Transformations
import io.reactivex.Flowable

class UserRepository(dataController: DataController,
                     private val api: CommentApiService,
                     private val gson: Gson,
                     private val commentSystem: CommentSystem): BaseRepository(dataController),
        UserRepositoryInterface {

    companion object {
        const val TAG = "UserRepo"
        const val COOLDOWN = 60 * 1000
        @JvmStatic val DEBUG = CommentSystem.getInstance().isDebug
    }

    private var likedMappings: Map<String, Int>? = null
    private var reportedPostUrls: Map<String, Int>? = null
    private var commentedPostUrls: Map<String, Int>? = null

    private val userStatusProcessor: UserStatusProcessor by lazy {
        UserStatusProcessor(dataController)
    }

    override fun mayFetchUserStatus(): Flowable<UserStatusQueryResult> {
        if (!commentSystem.isLoggedIn) {
            if (DEBUG) {
                Log.d(TAG, "not logged in")
            }
            return Flowable.just(UserStatusQueryResult(null, null, null))
        }

        val lastQueryTime = getLastUserStatusQueryTime()
        val diff = System.currentTimeMillis() - lastQueryTime
        if (diff <= COOLDOWN) {
            if (DEBUG) {
                Log.d(TAG, "within cooldown=$diff")
            }
            return Flowable.just(getCachedUserStatus())
        }

        return api.getUserStatus(dataController.appId)
                .compose(Transformations.flowableResultToResponseWithWithHttpErrorHandling())
                .flatMap{
                    val apiUserStatus = it.body()

                    if (apiUserStatus != null) {
                        userStatusProcessor.process(apiUserStatus)
                    } else {
                        Flowable.just(getCachedUserStatus())
                    }
                }
                .onErrorReturn {
                    getCachedUserStatus()
                }
    }

    override fun updateCommentLikeStatusOnCommentDbObject(likeMapping: Map<String, Int>?): Boolean {
        likeMapping?.run {
            likeMapping.forEach {
                dataController.updateCommentLikeStatus(it.key, it.value)
            }
            return true
        }
        return false
    }

    override fun getCommentLikeStatus(commentId: String): Int {
        /**
         * The type returned here is [Number]
         * @see [getLikeMapping] for more details
         */
        val number = getLikeMapping()[commentId]
        return number?.toInt() ?: 0
    }

    override fun isPostReported(url: String): Boolean {
//        Log.d("CUserRepository", "reported=${getReportedPostUrls()}")
        return getReportedPostUrls().containsKey(url)
    }

    override fun isPostCommented(url: String): Boolean {
//        Log.d("CUserRepository", "commented=${getCommentedPostUrls()}, dc=${dataController.getStringData(AppOptions.FIELD_COMMENTED_POST_LIKE_URLS)}")
        return getCommentedPostUrls().containsKey(url)
    }

    override fun addLikeOrDislikedComment(commentId: String, likeStatus: Int) {
        val map = (getLikeMapping() as ArrayMap)
        when (likeStatus) {
            1 -> map[commentId] = 1
            -1 -> map[commentId] = -1
            0 -> map.remove(commentId)
        }
        dataController.setStringData(AppOptions.FIELD_LIKE_MAPPING, gson.toJson(map))
    }

    override fun addCommentedPost(postUrl: String) {
        val map = getCommentedPostUrls() as ArrayMap
        map[postUrl] = 1
        dataController.setStringData(AppOptions.FIELD_COMMENTED_POST_LIKE_URLS, gson.toJson(map))
    }

    override fun addReportedPost(postUrl: String) {
        val map = getReportedPostUrls() as ArrayMap
        map[postUrl] = 1
        dataController.setStringData(AppOptions.FIELD_REPORTED_POST_URLS, gson.toJson(map))
    }

    override fun getLastUserStatusQueryTime(): Long {
        return dataController.getLongData(AppOptions.FIELD_LAST_USER_STATUS_QUERY_TIME)
    }

    override fun shouldOverrideCommentLikeStatus(): Boolean {
        val lastQueryTime = getLastUserStatusQueryTime()
        val should =  commentSystem.isLoggedIn && System.currentTimeMillis() - lastQueryTime >= COOLDOWN
        return should
    }

    override fun clear() {
        likedMappings?.let { (it as? ArrayMap)?.clear() }

        reportedPostUrls?.let { (it as? ArrayMap)?.clear() }

        commentedPostUrls?.let { (it as? ArrayMap)?.clear() }

        dataController.setLongData(AppOptions.FIELD_LAST_USER_STATUS_QUERY_TIME, 0)
    }

    /**
     * Not sure why after the fromJson call, the number being put into the ArrayMap is of the type
     * Double, so we use a more generic 'Number' type here, to cast it back to Int type when returned
     * Looks like a bug (?)
     */
    private fun convertMapToRealIntMap(tmpMap: Map<String, Number>?): Map<String, Int> {
        val realMap = ArrayMap<String, Int>()
        tmpMap?.forEach {
            realMap[it.key] = it.value.toInt()
        }
        return realMap
    }

    @VisibleForTesting
    internal fun getLikeMapping(): Map<String, Int> {
        if (likedMappings == null || likedMappings!!.isEmpty()) {
            synchronized(this) {
                val likeMappingJson = dataController.getStringData(AppOptions.FIELD_LIKE_MAPPING)
                val tmpLikeMapping = gson.fromJson<Map<String, Number>>(likeMappingJson, ArrayMap::class.java)
                likedMappings = convertMapToRealIntMap(tmpLikeMapping)
            }
        }
        return likedMappings!!
    }

    private fun getReportedPostUrls(): Map<String, Int> {
        if (reportedPostUrls == null || reportedPostUrls!!.isEmpty()) {
            synchronized(this) {
                val reportedPostJson = dataController.getStringData(AppOptions.FIELD_REPORTED_POST_URLS)
                val tmpMap =  gson.fromJson<Map<String, Number>>(reportedPostJson, ArrayMap::class.java)
                reportedPostUrls = convertMapToRealIntMap(tmpMap)
            }
        }
        return reportedPostUrls!!
    }

    private fun getCommentedPostUrls(): Map<String, Int> {
        if (commentedPostUrls == null || commentedPostUrls!!.isEmpty()) {
            synchronized(this) {
                val commentedPostJson = dataController.getStringData(AppOptions.FIELD_COMMENTED_POST_LIKE_URLS)
                val tmpMap = gson.fromJson<Map<String, Number>>(commentedPostJson, ArrayMap::class.java)
                commentedPostUrls = convertMapToRealIntMap(tmpMap)
            }
        }
        return commentedPostUrls!!
    }

    private fun getCachedUserStatus(): UserStatusQueryResult {
        return UserStatusQueryResult(getLikeMapping(), getCommentedPostUrls(), getReportedPostUrls())
    }

}
