package com.under9.android.comments.helper

import com.google.gson.Gson
import com.under9.android.comments.controller.DataController
import com.under9.android.comments.model.Comment
import com.under9.android.comments.model.CommentListItem
import com.under9.android.comments.model.api.ApiComment
import com.under9.android.comments.model.api.ApiTopComments
import com.under9.android.comments.GsonFactory

class CommentWriterV2(private val dataController: DataController,
                      private val gson: Gson) {


    private fun buildTopCommentsObject(json: String): ApiTopComments {
        return gson.fromJson(json, ApiTopComments::class.java)
    }

    fun writeTopCommentsToDB(url: String, order: Int, topCommentJSON: String): Boolean {
        dataController.run {
            if (! isAnyCommentExist(url, order)) {
                val topComments = buildTopCommentsObject(topCommentJSON) ?: return false

                val comments = topComments.comments

                if (comments.isEmpty()) return false

                Thread({
                    CommentHelper.processApiCommentList(comments, topComments.opUserId, null)
                    runInTx {
                        appendApiCommentsToDB(url, order, comments, null, 1, true)
                    }
                },"write-top-comment").start()

                return true
            }

            return false
        }
    }

    /**
     * Insert the given comment thread after the given comment tree 'index'
     * The left and right index of a comment indicate all nodes with left, right indices in between are children of it
     * E.g.
     *     L  R
     * A   1  6
     * A1  2  3
     * A2  4  5
     * B   7  8
     *
     * if startLeft is 6 (lte right index of a node )mean inserting comment thread as children of A
     * if startLeft is 9 (gt right index of a node)mean inserting the thread after B
     */
    fun appendApiCommentsToDB(url: String, order: Int,
                              comments: Array<ApiComment>?, parentComment: Comment?, startLeft: Int,
                              shouldOverrideLikeStatus: Boolean) {
        comments?.let {
            var currLeft = startLeft
            val n = it.size

            //TODO this cannot prevent duplicate update, if load-more is triggered twice, the left-right-index may be updated twice (which makes it wrong)
            /**
             * Since there will be n comments inserted in between with each ocuppying 2 indices (left and right), add n * 2 to later nodes to leave room for them
             **/
            dataController.updateTreeOrderAfter(url, order, startLeft, n*2)

            val dataHolder = arrayOfNulls<DataHolder>(n)

            //append this level first
            for (i in dataHolder.indices) {
                val apiComment = it[i]

                // https://fabric.io/9gag2/android/apps/com.ninegag.android.app/issues/55ef18bdf5d3a7f76be0841d
                if (apiComment.user == null) continue

                val user = dataController.obtainUser(apiComment.user)

                val comment = dataController.obtainComment(apiComment, parentComment, url, user, shouldOverrideLikeStatus)

                // assign two indices to each node
                val commentListItem = dataController.obtainCommentListItem(comment, apiComment.orderKey, url, order, currLeft, currLeft + 1)

                dataHolder[i] = DataHolder(comment, commentListItem)

                currLeft += 2
            }

            //append the children
            for (i in dataHolder.indices) {
                val apiComment = comments[i]
                val data = dataHolder[i]
                data!!.item.refresh()
                appendApiCommentsToDB(url, order, apiComment.children, data.comment, data.item.rgt, shouldOverrideLikeStatus)
            }
        }
    }

    private class DataHolder(val comment: Comment, val item: CommentListItem)

    companion object {
        val instance by lazy {
            CommentWriterV2(DataController.getInstance(), GsonFactory.getGson(GsonFactory.TYPE_STANDARD))
        }
    }
}