package com.under9.android.comments.model.ui

import android.content.Context
import android.graphics.Color
import android.graphics.drawable.*
import androidx.annotation.ColorInt
import androidx.annotation.DrawableRes
import androidx.collection.SparseArrayCompat
import androidx.core.content.ContextCompat
import com.under9.android.comments.CommentSystem
import com.under9.android.commentsystem.R
import com.under9.android.lib.util.ChatTimeFormatter
import com.under9.android.lib.util.TimeAgo
import com.under9.android.lib.util.TimeFormatter
import com.under9.android.lib.util.UiUtil
import com.under9.android.lib.widget.TextDrawable
import com.under9.android.theme.ThemeStore
import com.under9.android.theme.ThemeStoreProvider
import timber.log.Timber
import java.util.regex.Pattern

class CommentItemThemeAttr {
    companion object {
        @JvmField val mentionedUserPattern = Pattern.compile("@[a-zA-Z0-9_]+")
        // The url format is : [url]https://www.9gag.com[/url]
        @Deprecated("This pattern is not used anymore")
        @JvmField val urlPattern = Pattern.compile("\\[url\\][^\\[]+\\[\\/url\\]")

        @JvmField val urlPatternV2 = Pattern.compile("(http(s)?:\\/\\/.)?(www\\.)?[-a-zA-Z0-9@:%._\\+~#=]{2,256}\\.[a-z]{2,6}\\b([-a-zA-Z0-9@:%_\\+.~#?&//=]*)")
    }

    lateinit var opBackgroundDrawable: Drawable
        private set
    lateinit var highlightedColorStateListDrawable: StateListDrawable
        private set
    lateinit var selectedColorStateListDrawable: StateListDrawable
        private set

    private var hasInitedAttr = false
    lateinit var timeFormatter: TimeFormatter
        private set
    var timeFormatMode: Int = CommentSystem.TIME_FORMAT_MODE_ORIGINAL

    var avatarDrawable: Int = 0
        private set
    lateinit var colorArray: IntArray
        private set
    lateinit var textDrawableCache: Array<TextDrawable?>
        private set


    private val colorSparseArray = SparseArrayCompat<Int>()

    private fun createTimeFormatter(context: Context, timeFormatMode: Int): TimeFormatter {
        return when (timeFormatMode) {
            CommentSystem.TIME_FORMAT_MODE_CHAT -> ChatTimeFormatter(context)
            else -> TimeAgo(context)
        }
    }

    fun mayInitThemeAttr(context: Context) {
        if (hasInitedAttr) return

        timeFormatter = createTimeFormatter(context, timeFormatMode)

        val attrs = intArrayOf(
                R.attr.under9_themeForeground,
                R.attr.under9_themeTextColorPrimary,
                R.attr.under9_themeTextColorSecondary,
                R.attr.under9_themeTextColorPrimaryInverse,

                R.styleable.CommentSystemItemView_cs_themeTextColorPrimaryHighlighted,
                R.styleable.CommentSystemItemView_cs_themeTextColorSecondaryHighlighted,
                R.styleable.CommentSystemItemView_cs_themeForegroundHighlighted,
                R.styleable.CommentSystemItemView_cs_themeForegroundSelected,
                R.styleable.CommentSystemItemView_cs_themeTextColorOpHighlighted,

                R.attr.under9_themeColorAccent,
                R.attr.under9_themeLineColor,

                R.attr.cs_themeTextColorOpLabel,
                R.attr.cs_themeTextColorProBadgeLabel
        )

        if (context is ThemeStoreProvider) {
            assignColorsByThemeProvider((context as ThemeStoreProvider).getThemeStore(), context, attrs)
        } else {
            assignColorsByAppTheme(context, attrs)
        }

        val ta = context.obtainStyledAttributes(R.styleable.CommentSystemItemView)


        try {
            val selectedState = intArrayOf(android.R.attr.state_activated)
            val normalState = intArrayOf(android.R.attr.state_enabled)

            with(colorSparseArray) {
                highlightedColorStateListDrawable = StateListDrawable()
                highlightedColorStateListDrawable.addState(selectedState, ColorDrawable(get(R.styleable.CommentSystemItemView_cs_themeForegroundHighlighted, -1)))
                highlightedColorStateListDrawable.addState(normalState, ColorDrawable(get(R.attr.under9_themeForeground, -1)))

                selectedColorStateListDrawable = StateListDrawable()
                selectedColorStateListDrawable.addState(selectedState, ColorDrawable(get(R.styleable.CommentSystemItemView_cs_themeForegroundSelected, -1)))
                selectedColorStateListDrawable.addState(normalState, ColorDrawable(get(R.attr.under9_themeForeground, -1)))

                // OP's shape
                val cornerRadius = UiUtil.dpToPx(context, 2).toFloat()
                val shape = GradientDrawable()
                shape.colors = intArrayOf(get(R.attr.under9_themeColorAccent, -1), get(R.attr.under9_themeColorAccent, -1))
                shape.shape = GradientDrawable.RECTANGLE
                shape.cornerRadius = cornerRadius
                opBackgroundDrawable = shape

                colorArray = context.resources.getIntArray(R.array.under9_rainbow)
                textDrawableCache = arrayOfNulls(colorArray.size)
                avatarDrawable = ta.getResourceId(R.styleable.CommentSystemItemView_cs_themeAvatarDrawable, -1)
            }

        } finally {
            ta.recycle()
            hasInitedAttr = true
        }
    }

    fun getColor(attr: Int): Int = colorSparseArray.get(attr, -1)

    private fun assignColorsByAppTheme(context: Context, colorInts: IntArray) {
        val ta = context.obtainStyledAttributes(R.styleable.CommentSystemItemView)

        for (i in 0 until colorInts.size) {
            when (val attr = colorInts[i]) {
                R.styleable.CommentSystemItemView_cs_themeTextColorPrimaryHighlighted,
                R.styleable.CommentSystemItemView_cs_themeTextColorSecondaryHighlighted,
                R.styleable.CommentSystemItemView_cs_themeForegroundHighlighted,
                R.styleable.CommentSystemItemView_cs_themeForegroundSelected,
                R.styleable.CommentSystemItemView_cs_themeTextColorOpHighlighted -> {
                    colorSparseArray.put(attr, ta.getColor(attr, ContextCompat.getColor(context, R.color.under9_theme_red)))
                }
                else -> {
                    colorSparseArray.put(attr, UiUtil.themeAttributeToColor(attr, context, -1))
                }
            }
        }

        ta.recycle()

        Timber.d("assignColorsByAppTheme")
    }

    private fun assignColorsByThemeProvider(themeStore: ThemeStore, context: Context, colorInts: IntArray) {
        val ta = context.obtainStyledAttributes(R.styleable.CommentSystemItemView)

        for (i in 0 until colorInts.size) {
            when (val attr = colorInts[i]) {
                R.styleable.CommentSystemItemView_cs_themeTextColorPrimaryHighlighted,
                R.styleable.CommentSystemItemView_cs_themeTextColorSecondaryHighlighted,
                R.styleable.CommentSystemItemView_cs_themeForegroundHighlighted,
                R.styleable.CommentSystemItemView_cs_themeForegroundSelected,
                R.styleable.CommentSystemItemView_cs_themeTextColorOpHighlighted -> {
                    colorSparseArray.put(attr, ta.getColor(attr, ContextCompat.getColor(context, R.color.under9_theme_red)))
                }
                else -> {
                    colorSparseArray.put(attr, themeStore.getColor(attr))
                }
            }
        }

        ta.recycle()

        Timber.d("assignColorsByThemeProvider, themeStore=$themeStore")
    }
}