package com.under9.android.comments.model.wrapper

import android.content.Context
import android.os.Bundle
import android.text.Html
import android.text.Spanned
import android.text.TextUtils
import androidx.collection.ArrayMap
import com.under9.android.comments.CommentSystem
import com.under9.android.comments.data.repository.AppInfoRepositoryInterface
import com.under9.android.comments.model.Comment
import com.under9.android.comments.model.CommentListItem
import com.under9.android.comments.model.PendingCommentListItem
import com.under9.android.comments.model.constant.CommentConstant
import com.under9.android.commentsystem.R
import com.under9.android.lib.util.GsonUtil
import com.under9.android.lib.util.L10nUtil
import com.under9.android.lib.util.TimeAgo
import com.under9.android.lib.util.TimeFormatter
import com.under9.android.lib.widget.media.overlayv3.IOverlayable
import timber.log.Timber

class CommentWrapperV2 : IOverlayable, CommentWrapperInterface {

    private val TAG = "CommentWrapperV2"
    private val DEBUG = CommentSystem.getInstance().isDebug
    private val NEW_INTERVAL = 5000
    private val FORCE_HIGHLIGHT_INTERVAL = NEW_INTERVAL

    private var forceShouldHighlight = false

    private var pendingCommentListItem: PendingCommentListItem? = null
    private var commentListItem: CommentListItem? = null
    private var comment: Comment
    // For optimization. But not using lateinit here as
    // it needs to check if it's been inited everytime when you access it
    // so just let it to be nullable
    private var children: MutableList<CommentWrapperV2>? = null

    private var data: Bundle? = null
    private var highlightStartTime: Long = 0

    /**
     * Note: Parent comment's database ID, not the ID in API
     */
    private var parentId: Long = 0
    private var isPending: Boolean = false
    private var commentLabel: String? = null

    private var cachedContent: Spanned? = null
    private var cachedRichtext: Spanned? = null
    private var cachedMediaText: Spanned? = null

    private var embedMediaMeta: EmbedMediaMeta? = null
    var media: Array<Media>? = null
        private set

    private var mentionMap: Map<String, String> = ArrayMap()
    private var suppData: SuppData? = null
    private var localCollapsed = false

    private var isUrl: Boolean = false

    var mediaText: String? = ""
        get() = comment.mediaText
        private set

    constructor(commentListItem: CommentListItem, highlightCommentId: String? = null) {
        this.commentListItem = commentListItem
        this.comment = commentListItem.comment
        this.comment.let { comment ->
            for (i in 0 until comment.childrenComments.size) {
                if (children == null) {
                    children = ArrayList()
                }
                val child = CommentWrapperV2(comment.childrenComments[i])
                this.children?.add(child)

                if (child.getCommentId() == highlightCommentId) {
                    child.forceShouldHighlight = true
                }
            }
        }

        init()
    }

    constructor(pendingCommentListItem: PendingCommentListItem) {
        this.isPending = true
        this.pendingCommentListItem = pendingCommentListItem
        this.comment = pendingCommentListItem.comment
        this.data = Bundle()
        this.comment.let { comment ->
            comment.user?.let { user -> data?.putString("user_id", user.userId) }
        }

        init()
    }

    constructor(comment: Comment) {
        this.comment = comment
        init()
    }

    private fun init() {
        try {
            embedMediaMeta =
                    GsonUtil.fromJson(comment.embedMediaMeta ?: "", EmbedMediaMeta::class.java)
            media = GsonUtil.fromJson(comment.media ?: "", Array<Media>::class.java)
            val mapping: String = comment.mentionMapping ?: ""
            suppData = GsonUtil.fromJson(comment.suppData ?: "", SuppData::class.java)
            if (!mapping.isEmpty()) {
                val map: ArrayMap<String, String> = GsonUtil.getGson().fromJson(mapping, CommentWrapperHelper.mapTypeToken)
                        ?: return
                mentionMap = map
            }
        } catch (e: Exception) {

        }
    }

    override fun getType(): Int {
        return comment.type ?: CommentConstant.TYPE_TEXT
    }

    override fun getId(): Long {
        return comment.id ?: 0
    }

    override fun getCommentId(): String {
        return comment.commentId ?: ""
    }

    fun getUserId(): String? {
        comment.let { comment ->
            return comment.user?.userId
        }
    }

    override fun getMentionedAccountId(username: String): String? {
        return mentionMap[username]
    }

    override fun getComment(): Comment? {
        return this.comment
    }

    // FIXME: 7/1/2019
    override fun getParentComment(): Comment? {
        return this.comment.parentComment
    }

    override fun getLevel(): Int {
        return this.comment.nestingLevel ?: 0
    }

    override fun getLevel(appInfoRepository: AppInfoRepositoryInterface): Int {
        return Math.min(this.comment?.nestingLevel ?: 0, appInfoRepository.getMaxCommentLevel())
    }

    override fun isPending(): Boolean {
        return this.isPending
    }

    override fun getEmbedMediaMeta(): EmbedMediaMeta? {
        return this.embedMediaMeta
    }

    override fun getLikeCount(): Int {
        return comment.likeCount ?: 0
    }

    override fun getLikeStatus(): Int {
        return comment.isLike ?: 0
    }

    override fun getDislikeCount(): Int {
        return comment.dislikeCount ?: 0
    }

    override fun getDislikeStatus(): Int {
        val isLike = comment.isLike ?: return 0

        return when {
            isLike > 0 -> -1 // if liked
            isLike < 0 -> 1 // if disliked
            else -> 0
        }
    }

    override fun getPoints(): Int {
        return comment.likeCount ?: 0
    }

    override fun getTime(): Long {
        Timber.d("comment?.timestamp =${(comment.timestamp ?: 0).toLong() * 1000}" )
        return (comment.timestamp ?: 0).toLong() * 1000
    }

    override fun getDisplayName(): String {
        return comment.user?.displayName ?: ""
    }

    override fun getUrl(): String? {
        return comment.url
    }

    fun getChildren(): MutableList<CommentWrapperV2> {
        if (this.children == null) {
            this.children = ArrayList()
        }
        return this.children!!
    }

    override fun getAvatarUrl(): String? {
        comment.let { comment ->
            return comment.user?.avatarUrl
        }
    }

    override fun getCommentLabel(): String? {
        comment.let { comment -> return comment.commentLabel }
    }

    override fun getStatus(): Int {
        return comment.status ?: 0
    }

    override fun getUserName(context: Context): String {
        if (isPending) {
            if (comment.user == null) {
                return context.getString(R.string.me)
            }
        }

        return comment.user?.displayName ?: context.getString(R.string.anonymous)
    }

    override fun getUserEmojiStatus(): String {
        if (isPending) {
            if (comment.user == null) {
                return ""
            }
        }
        return (if (comment.user == null) "" else comment.user?.emojiStatus) ?: return ""
    }

    override fun getUserCountryEmoji(): String {
        if (isPending) {
            if (comment.user == null) {
                return ""
            }
        }
        return (if (comment.user == null) "" else L10nUtil.countryCodeToEmoji(comment.user?.country ?: "")) ?: return ""
    }

    fun refreshCommentData(): Boolean {
        comment.let { comment ->
            return try {
                comment.refresh()
                init()
                true
            } catch (e: Exception) {
                false
            }
        }
    }

    override fun getContent(): String? {
        return comment.commentText
    }

    override fun getHtmlContent(): Spanned? {
        if (cachedContent != null) return cachedContent

        var content = getContent()
        if (TextUtils.isEmpty(content)) {
            content = ""
        }
        cachedContent = Html.fromHtml(content!!.replace("\n", "<br/>"))

        return cachedContent
    }

    fun getRichtext(): String? {
        return comment.richtext

    }

    override fun getHtmlRichtext(): Spanned? {
        if (cachedRichtext != null) return cachedRichtext

        cachedRichtext = Html.fromHtml(getRichtext()!!.replace("\n", "<br/>"))

        return cachedRichtext
    }

    override fun isRichtextAvailable(): Boolean {
        return getRichtext() != null
    }

    fun refreshPendingItem(): Boolean {
        return try {
            pendingCommentListItem?.let { pendingCommentListItem ->
                pendingCommentListItem.refresh()
                comment = pendingCommentListItem.comment
                comment.refresh()
                init()

                return true
            }

            false
        } catch (e: Exception) {
            false
        }
    }

    override fun isAnonymous(): Boolean {
        return CommentWrapperHelper.isAnonymous(comment)
    }

    override fun getTimeText(timeFormatter: TimeFormatter): String {
        if (isPending) {
            if (pendingCommentListItem?.status != CommentSystem.PENDING_STATUS_SENT) {
                return "..."
            }
        }
        return timeFormatter.formatTime(getTime())
    }

    override fun shouldHighlight(): Boolean {
        return forceShouldHighlight && !isForceHighlightTimePassed() || isNew()
    }

    fun setForceHighlight(highlight: Boolean) {
        this.forceShouldHighlight = highlight
    }

    fun isNew(): Boolean {
        return if (isPending) System.currentTimeMillis() - (pendingCommentListItem?.createTime ?: 0) < NEW_INTERVAL else false
    }

    override fun isMediaTextAvailable(): Boolean {
        return comment.mediaText?.isNotEmpty() ?: false
    }

    override fun getHtmlMediaText(): Spanned? {
        if (cachedMediaText != null) return cachedMediaText
        cachedMediaText = Html.fromHtml(mediaText)
        return cachedMediaText
    }

    override fun getFirstMediaEmbedImage(): EmbedImage? {
        return if (media == null || media?.size == 0) null else media!![0].imageMetaByType
    }

    override fun isUrl(): Boolean {
        return comment.isUrl ?: 0 == 1
    }

    private fun isForceHighlightTimePassed(): Boolean {
        return if (highlightStartTime == 0L) {
            //not yet highlighted before, not yet passed then
            false
        } else System.currentTimeMillis() - highlightStartTime >= FORCE_HIGHLIGHT_INTERVAL
    }

    override fun toString(): String {
        return "content={${getContent()}}\n " +
                "numOfChildren=${children?.size}\n" +
                "children=$children, " +
                super.toString()
    }

    override fun getIsProUser() = comment.user?.let{ it.isActivePro } ?: false

    override fun getIsProPlusUser() = comment.user?.let { it.isActiveProPlus } ?: false

    override fun isUserActive(): Boolean {
        if (isPending) return true
        return (System.currentTimeMillis() - ((comment.user?.activeTs ?: 0) * 1000)) <= CommentSystem.getInstance().config.activeDuration
    }

}