package com.under9.android.comments.ui.fragment.singlecomment

import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleObserver
import androidx.lifecycle.OnLifecycleEvent
import com.hwangjr.rxbus.annotation.Subscribe
import com.jakewharton.rxrelay2.PublishRelay
import com.ninegag.android.blitz2.Blitz
import com.ninegag.android.blitz2.BlitzState
import com.ninegag.android.blitz2.ui.delegate.BlitzViewStateCallback
import com.ninegag.android.blitz2.ui.delegate.NoOpViewStateCallback
import com.under9.android.comments.CommentSystem
import com.under9.android.comments.controller.CommentSystemController
import com.under9.android.comments.controller.CommentSystemTaskQueueController
import com.under9.android.comments.controller.DataController
import com.under9.android.comments.data.query.comment.CommentListQueryParam
import com.under9.android.comments.data.query.comment.CommentListQueryResult
import com.under9.android.comments.data.repository.AppInfoRepositoryInterface
import com.under9.android.comments.data.repository.CommentListRepositoryInterface
import com.under9.android.comments.data.repository.UserRepositoryInterface
import com.under9.android.comments.event.DeleteCommentEvent
import com.under9.android.comments.event.HideCommentEvent
import com.under9.android.comments.event.RemoveCommentEvent
import com.under9.android.comments.event.ReportCommentDoneEvent
import com.under9.android.comments.event.ReportCommentEvent
import com.under9.android.comments.event.SelectCommentEvent
import com.under9.android.comments.model.Comment
import com.under9.android.comments.model.wrapper.CommentWrapperInterface
import com.under9.android.comments.model.wrapper.CommentWrapperV2
import com.under9.android.lib.core.mvvm.RxViewModel
import io.reactivex.Flowable
import io.reactivex.Single
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import io.reactivex.schedulers.Schedulers
import timber.log.Timber

open class SingleSublevelCommentListViewModel(
        private val url: String,
        private val commentSystem: CommentSystem,
        private val userRepository: UserRepositoryInterface,
        private val appInfoRepository: AppInfoRepositoryInterface,
        private val commentListRepository: CommentListRepositoryInterface,
        private val commentSystemActionListener: CommentSystem.CommentSystemActionListener?,
        private val dataController: DataController,
        private val queryParam: CommentListQueryParam
) : RxViewModel(), LifecycleObserver {

    val commentSystemTaskQueueController by lazy(LazyThreadSafetyMode.NONE) {
        CommentSystemTaskQueueController(commentSystem.context, commentSystem.authHash)
    }

    val commentSystemController by lazy(LazyThreadSafetyMode.NONE) {
        CommentSystemController(commentSystem.context, commentSystemTaskQueueController, userRepository, appInfoRepository)
    }

    val removeCommentRelay = PublishRelay.create<String>()
    val selectCommentRelay = PublishRelay.create<SelectCommentEvent>()
    val reportedCommentRelay = PublishRelay.create<CommentWrapperInterface>()
    val reportCommentDoneRelay = PublishRelay.create<String>()
    val blitzViewStateRelay = PublishRelay.create<@Blitz.State Int>()
    val queryResultRelay = PublishRelay.create<QueryResult>()
    lateinit var commentWrappers: CommentWrapperV2
    private set
    var queryResult: QueryResult? = null

    open fun loadNext() {
        blitzViewStateRelay.accept(BlitzState.STATE_LOADING_MORE)
        queryResult?.let {
            val nextQueryParam = CommentListQueryParam(
                    isRefresh = false,
                    url = queryParam.url,
                    order = queryParam.order,
                    count = queryParam.count,
                    ref = it.nextRef,
                    refCommentId = it.nextRef,
                    parentCommentId = queryParam.parentCommentId,
                    direction = queryParam.direction,
                    commentId = queryParam.commentId,
                    shouldOverrideLikeStatus = queryParam.shouldOverrideLikeStatus,
                    level = queryParam.level
            )

            Timber.d("onSublevelLoadMore: e=parentCommentId=${nextQueryParam.parentCommentId}, ref=${it.nextRef}")

            disposables.add(commentListRepository.getCommentList(nextQueryParam)
                    .map { result -> this.processComment(result, nextQueryParam, isLoadNext = true) }
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe({ result ->
                        queryResultRelay.accept(result)
                        if (result.hasNext) {
                            blitzViewStateRelay.accept(BlitzState.STATE_FINISH_LOADING_HAS_NEXT)
                        } else {
                            blitzViewStateRelay.accept(BlitzState.STATE_FINISH_LOADING_NEXT_END_OF_LIST)
                        }
                    }, Timber::e))
        }
    }

    fun loadInitial() {
        blitzViewStateRelay.accept(BlitzState.STATE_INIT_LOADING)
        Timber.d("blitzView.Init")
        val commentListQueryParam = queryParam
        disposables.add(createCommentLoadDisposable(commentListQueryParam))
    }

    fun refreshComment() {
        val refreshQueryParam = CommentListQueryParam(
                isRefresh = true,
                url = queryParam.url,
                order = queryParam.order,
                count = queryParam.count,
                ref = queryParam.ref,
                refCommentId = queryParam.refCommentId,
                parentCommentId = queryParam.parentCommentId,
                direction = queryParam.direction,
                commentId = queryParam.commentId,
                shouldOverrideLikeStatus = queryParam.shouldOverrideLikeStatus,
                level = 1
        )
        disposables.add(createCommentLoadDisposable(refreshQueryParam))
    }

    fun getBlitzViewStateCallback(): BlitzViewStateCallback {
        return NoOpViewStateCallback()
    }

    private fun createCommentLoadDisposable(commentListQueryParam: CommentListQueryParam): Disposable {
        val base = if (commentListQueryParam.isRefresh) {
            Flowable.fromCallable {
                dataController.getCommentListItemByCommentId(commentListQueryParam.parentCommentId, commentListQueryParam.url, commentListQueryParam.order)
            }.flatMap {
                commentListRepository.getCommentList(commentListQueryParam)
            }.doOnError {
                Timber.e(it)
            }
        } else {
            Flowable.defer {
                Flowable.just(CommentListQueryResult(true, dataController.getLastOrderKey(commentListQueryParam.url, commentListQueryParam.order)))
                        .concatWith(commentListRepository.getCommentList(commentListQueryParam))
            }
        }

        return base.map { this.processComment(it, commentListQueryParam) }
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe {
                    Timber.d("queryResult=size=${it.commentWrapper?.getChildren()?.size}")
                    it.commentWrapper?.let { wrapper ->
                        commentWrappers = wrapper

                        queryResultRelay.accept(it)
                        if (it.hasNext) {
                            blitzViewStateRelay.accept(BlitzState.STATE_FINISH_LOADING_HAS_NEXT)
                        } else {
                            blitzViewStateRelay.accept(BlitzState.STATE_FINISH_LOADING_NEXT_END_OF_LIST)
                        }
                    }

                    if (it.commentWrapper == null) {
                        commentSystem.logger.log("SUBLEVEL_COMMENT_NULL", "DEBUG", "$commentListQueryParam")
                    }
                }
    }

    private fun processComment(commentListQueryResult: CommentListQueryResult,
                               commentListQueryParam: CommentListQueryParam,
                               isLoadNext: Boolean = false): QueryResult {

        Timber.d("queryResult=$commentListQueryResult, param=$commentListQueryParam")
        dataController.clearAllCache()
        with(commentListQueryParam) {
            val commentListItem = dataController.getCommentListItemByCommentId(parentCommentId, url, order)
            val pendingCommentListItem  = dataController.getPendingComments(url)
            var wrapper: CommentWrapperV2? = null
            var lastComment: Comment? = null
            if (commentListItem != null) {
                lastComment = if (commentListItem.comment.childrenComments.isEmpty()) {
                    commentListItem.comment
                } else {
                    commentListItem.comment.childrenComments.last()
                }
                wrapper = CommentWrapperV2(commentListItem, commentListQueryParam.commentId)
            } else if (pendingCommentListItem != null && pendingCommentListItem.isNotEmpty()) {
                pendingCommentListItem.forEachIndexed { index, item ->
                    val comment = item.comment
                    Timber.d("idx=$index, pendingComment=${comment.commentText}, id=${comment.commentId}, queryId=${commentId}")
                    // QueryParam's id
                    if (item.status == CommentSystem.PENDING_STATUS_PENDING && comment.commentId == ref) {
                        wrapper = CommentWrapperV2(pendingCommentListItem[index])
                        return@forEachIndexed
                    }
                }
            }


            Timber.d("lastComment=$lastComment, lastCommentHasNext=${lastComment?.hasNext}, commentListQueryResult=$commentListQueryResult")

            queryResult = QueryResult(
                    isRefresh = isRefresh,
                    nextRef = lastComment?.commentId ?: "",
                    hasNext = commentListQueryResult.hasNext,
                    commentWrapper = wrapper,
                    isLoadNext = isLoadNext
            )
        }

        return queryResult!!
    }

    fun listHasNext(): Boolean {
        return queryResult?.hasNext ?: false
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_START)
    fun subscribe() {
        CommentSystem.registerEventBus(url, this)
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_STOP)
    fun unsubscribe() {
        CommentSystem.unregisterEventBus(url, this)
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_DESTROY)
    override fun onCleared() {
        super.onCleared()
    }

    @Subscribe
    fun onReportedComment(e: ReportCommentEvent) {
        Single.just(e.commentId)
                .subscribeOn(Schedulers.io())
                .map {
                    val comment = dataController.getCommentByCommentId(e.commentId)
                    val commentWrapper = CommentWrapperV2(comment)
                    commentWrapper
                }
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({
                    reportedCommentRelay.accept(it)
                }, Timber::w)
    }

    @Subscribe
    fun onRemoveComment(e: RemoveCommentEvent) {
        removeCommentRelay.accept(e.commentId)
    }

    @Subscribe
    fun onReportCommentDone(e: ReportCommentDoneEvent) {
        reportCommentDoneRelay.accept(e.commentId)
    }

    @Subscribe
    fun onDeletedComment(e: DeleteCommentEvent) {
        removeCommentRelay.accept(e.commentId)
    }

    @Subscribe
    fun onHideComment(e: HideCommentEvent) {
        removeCommentRelay.accept(e.commentId)
    }

    @Subscribe
    fun onSelectComment(e: SelectCommentEvent) {
        selectCommentRelay.accept(e)
    }

    fun isCommentWrapperInited() = ::commentWrappers.isInitialized
}