package com.under9.android.comments.ui.renderer

import android.net.Uri
import android.text.SpannableStringBuilder
import android.text.method.LinkMovementMethod
import android.view.View
import android.widget.FrameLayout
import androidx.appcompat.content.res.AppCompatResources
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.core.view.ViewCompat
import androidx.recyclerview.widget.RecyclerView
import com.under9.android.comments.CommentSystem
import com.under9.android.comments.controller.DataController
import com.under9.android.comments.data.repository.AppInfoRepositoryInterface
import com.under9.android.comments.model.constant.CommentConstant
import com.under9.android.comments.model.ui.CommentItemThemeAttr
import com.under9.android.comments.model.wrapper.CommentWrapperHelper
import com.under9.android.comments.model.wrapper.CommentWrapperInterface
import com.under9.android.comments.ui.widget.CommentItemView
import com.under9.android.commentsystem.R
import com.under9.android.lib.util.ColorUtils
import com.under9.android.lib.util.UiUtil
import com.under9.android.theme.ThemeStoreProvider
import kotlinx.android.synthetic.main.comment_system_list_item_inner_v5.view.*
import me.saket.bettermovementmethod.BetterLinkMovementMethod
import timber.log.Timber

class CommentItemRendererDelegate(
        private val appInfoRepository: AppInfoRepositoryInterface,
        private val dataController: DataController,
        private val commentItemActionDelegate: CommentItemActionDelegate,
        private val indentationLevel: Int,
        private val avatarMode: Int = CommentSystem.AVATAR_MODE_IMAGE,
        private val shouldHighlightUrlInComment: Boolean = false,
        private val shouldShowCountry: Boolean = false
) {

    private var dp16: Int = 0
    private var rootAvatarSize: Int = 0
    private var subLevelAvatarSize: Int = 0
    private var opUsernamePadding: Int = 0
    private var opUsernamePaddingTopBottom: Int = 0
    private val clickListener = CommentItemClickListener(
            commentItemActionDelegate,
            appInfoRepository, true)

    private lateinit var linkMovementMethod: LinkMovementMethod

    fun applyDrawableStates(viewHolder: RecyclerView.ViewHolder) {
        with(viewHolder.itemView) {
            // FIXME: 4/1/2019 Needs optimization here
            //<editor-fold desc="FIXME Optimization needed">
            likeBtn.buttonDrawable = AppCompatResources.getDrawable(context, R.drawable.ic_comment_upvote_v2)
            dislikeBtn.buttonDrawable = AppCompatResources.getDrawable(context, R.drawable.ic_comment_downvote_v2)

            if (context is ThemeStoreProvider) {
                likeBtn.buttonDrawable = ColorUtils.getTintedVectorDrawable(context,
                        R.drawable.ic_comment_upvote_v2,
                        (context as ThemeStoreProvider).getThemeStore().checkedColorStateList)

                dislikeBtn.buttonDrawable = ColorUtils.getTintedVectorDrawable(context,
                        R.drawable.ic_comment_downvote_v2,
                        (context as ThemeStoreProvider).getThemeStore().checkedColorStateList)
            }
        }
    }

    fun applyDepth(wrapper: CommentWrapperInterface, viewHolder: RecyclerView.ViewHolder) {
        with(viewHolder.itemView) {
            if (rootAvatarSize <= 0) {
                rootAvatarSize = resources.getDimensionPixelSize(R.dimen.avatar_size)
            }

            if (dp16 <= 0) {
                dp16 = UiUtil.dpToPx(context, 16)
            }

            val level = wrapper.getLevel() + indentationLevel - 1
            val avatarSize = (rootAvatarSize * Math.pow(0.6, (level).toDouble())).toInt()
            val marginLeft = dp16 * (level)

            val layoutParams = avatar?.layoutParams as ConstraintLayout.LayoutParams
            layoutParams.setMargins(marginLeft, layoutParams.topMargin, layoutParams.rightMargin, layoutParams.bottomMargin)
            layoutParams.width = avatarSize
            layoutParams.height = avatarSize

            val avatarLayoutParams = avatar?.avatar?.layoutParams as FrameLayout.LayoutParams
            avatarLayoutParams.width = avatarSize
            avatarLayoutParams.height = avatarSize
        }
    }

    fun applyUserInfo(wrapper: CommentWrapperInterface, viewHolder: RecyclerView.ViewHolder, themeAttr: CommentItemThemeAttr) {
        var avatarUrl: String?
        with(viewHolder.itemView as CommentItemView) {
            val username: String
            if (wrapper.isAnonymous()) {
                if (CommentWrapperHelper.isCommentCreator(wrapper.getComment(), dataController.loggedInUserId)) {
                    username = context.getString(R.string.my_anonymous_comment_display_name)
                    userName.text = username
                } else {
                    username = context.getString(R.string.anonymous_comment_display_name)
                    userName.text = username
                }
                avatarUrl = appInfoRepository.getAnonymousAvatarUrl()
                userName.setOnClickListener(null)
                avatar.setOnClickListener(null)
            } else {
                username = wrapper.getUserName(context)
                val emojiStatus = wrapper.getUserEmojiStatus()
                val country = if (shouldShowCountry) wrapper.getUserCountryEmoji() else ""

                userName.text = if (emojiStatus.isEmpty() && country.isEmpty()) {
                    username
                } else if (emojiStatus.isEmpty() && country.isNotEmpty()) {
                    "$username $country"
                } else if (emojiStatus.isNotEmpty() && country.isEmpty()) {
                    "$username $emojiStatus"
                } else {
                    "$username $country $emojiStatus"
                }
                userName.setTag(R.id.username, username)

                avatarUrl = wrapper.getAvatarUrl()
                avatar.setTag(R.id.username, username)

                // apply the pro badge if that user is pro user
                if (wrapper.getIsProUser()) {
                    val isProPlus = wrapper.getIsProPlusUser()
                    proBadge.updateProBadge(isProPlus = isProPlus)
                    proBadge.visibility = View.VISIBLE
                } else {
                    proBadge.visibility = View.GONE
                }
            }

            when (avatarMode) {
                CommentSystem.AVATAR_MODE_IMAGE -> avatar.setImageURI(if (avatarUrl == null) null else Uri.parse(avatarUrl))
                else -> {
                    setRoundAvatarColorDrawable(themeAttr.colorArray, themeAttr.textDrawableCache, themeAttr.avatarDrawable, username)
                }
            }

            avatar.isActive = CommentWrapperHelper.isCommentCreator(wrapper.getComment(), dataController.loggedInUserId) ||
                    wrapper.isUserActive()
        }
    }

    fun applyOpInfo(wrapper: CommentWrapperInterface, viewHolder: RecyclerView.ViewHolder, themeAttr: CommentItemThemeAttr) {
        with(viewHolder.itemView) {
            val showOP = CommentWrapperHelper.isOP(wrapper.getComment()) && appInfoRepository.isEnableOpLabelFeature()

            if (opUsernamePadding <= 0) {
                opUsernamePadding = resources.getDimensionPixelSize(R.dimen.space4)
            }

            if (opUsernamePaddingTopBottom <= 0) {
                opUsernamePaddingTopBottom = resources.getDimensionPixelSize(R.dimen.cs_metaTextTopBottomPadding)
            }

            if (showOP) {
                // TODO: 18/2/2019 Highlight OP
                userName.setTextColor(themeAttr.getColor(R.styleable.CommentSystemItemView_cs_themeTextColorOpHighlighted))
                ViewCompat.setBackground(userName, themeAttr.opBackgroundDrawable)
                userName.setPadding(opUsernamePadding, opUsernamePaddingTopBottom, opUsernamePadding, opUsernamePaddingTopBottom)
            } else {
                // Don't highlight OP
                userName.setTextColor(themeAttr.getColor(R.attr.under9_themeTextColorPrimary))
                userName.setPadding(0, opUsernamePaddingTopBottom, 0, opUsernamePaddingTopBottom)
                ViewCompat.setBackground(userName, null)
            }
        }
    }

    fun applyTextContent(wrapper: CommentWrapperInterface,
                         viewHolder: RecyclerView.ViewHolder, themeAttr: CommentItemThemeAttr) {
        with(viewHolder.itemView) {
            val textContent = getTextContent(wrapper)
            if (textContent == null || textContent.isEmpty()) {
                content.visibility = View.GONE
            } else {
                content.visibility = View.VISIBLE
            }

            // mention user support
            // TODO: 3/1/2019
            val contentBuilder = SpannableStringBuilder(textContent)
            if (appInfoRepository.mobileUsernameClickable()) {
                val matcher = CommentItemThemeAttr.mentionedUserPattern.matcher(textContent)
                while (matcher.find()) {
                    commentItemActionDelegate.highlightMentionedUser(contentBuilder,
                            matcher.start(), matcher.end(), matcher.group(),
                            wrapper)
                }
            }

            // image or other url support
            Timber.d("isUrl: ${wrapper.isUrl()}, url=${wrapper.getUrl()}")
            if (shouldHighlightUrlInComment) {
                val matcher = CommentItemThemeAttr.urlPatternV2.matcher(textContent)
                while (matcher.find()) {
                    commentItemActionDelegate.highlightUrlV2(contentBuilder, matcher.start(), matcher.end(), matcher.group(),
                            wrapper)
                }
            }

            content?.text = contentBuilder
            content?.movementMethod = getLinkMovementMethod(wrapper.getUrl() ?: "")
        }
    }

    fun applyContentTheme(wrapper: CommentWrapperInterface, holder: RecyclerView.ViewHolder, themeAttr: CommentItemThemeAttr) {
        with(holder.itemView) {
            isActivated = when {
                commentItemActionDelegate.isSelectedComment(wrapper.getId()) -> {
                    ViewCompat.setBackground(this, themeAttr.selectedColorStateListDrawable)
                    true
                }
                wrapper.shouldHighlight() -> {
                    ViewCompat.setBackground(this, themeAttr.highlightedColorStateListDrawable)
                    true
                }
                else -> {
                    this.setBackgroundColor(themeAttr.getColor(R.attr.under9_themeForeground))
                    false
                }
            }

            commentBorder.setBackgroundColor(themeAttr.getColor(R.attr.under9_themeLineColor))
//            commentLabel.setTextColor(themeAttr.opLabelColor)

            if (wrapper.shouldHighlight()) {
//                userName.setTextColor(themeAttr.highlightedTextColorPrimary)

                meta.setTextColor(themeAttr.getColor(R.styleable.CommentSystemItemView_cs_themeTextColorSecondaryHighlighted))
                likeBtn?.setTextColor(themeAttr.getColor(R.styleable.CommentSystemItemView_cs_themeTextColorSecondaryHighlighted))
                dislikeBtn?.setTextColor(themeAttr.getColor(R.styleable.CommentSystemItemView_cs_themeTextColorSecondaryHighlighted))

                // TODO transition

            } else {
//                userName.setTextColor(themeAttr.textColorPrimary)
                meta.setTextColor(themeAttr.getColor(R.attr.under9_themeTextColorSecondary))
                commentItemActionDelegate.setLikeCount(context, likeBtn, wrapper)
                commentItemActionDelegate.setDislikeCount(context, dislikeBtn, wrapper)
            }
        }
    }

    fun applyTextTheme(wrapper: CommentWrapperInterface,
                       holder: RecyclerView.ViewHolder, themeAttr: CommentItemThemeAttr) {
        with(holder.itemView) {
            if (wrapper.shouldHighlight()) {
                content.setTextColor(themeAttr.getColor(R.styleable.CommentSystemItemView_cs_themeTextColorPrimaryHighlighted))
            } else {
                content.setTextColor(themeAttr.getColor(R.attr.under9_themeTextColorPrimary))
            }
        }
    }

    fun getTextContent(wrapper: CommentWrapperInterface): String? {
        val commentText: String? = wrapper.getComment()?.commentText

        when (wrapper.getType()) {
            CommentConstant.TYPE_MEDIA, CommentConstant.TYPE_USER_MEDIA -> {
                if (wrapper.isUrl() && wrapper.getComment() != null) {
                    if (commentText != null &&
                            (commentText.contains(DefaultCommentItemItemBinder.MEMEFUL_URL) || wrapper.getType() == CommentConstant.TYPE_MEDIA)
                    ) {
                        return ""
                    }
                }
                return if (wrapper.isMediaTextAvailable()) wrapper.getHtmlMediaText().toString() else ""
            }
            else -> {

                return if (wrapper.isRichtextAvailable()) {
                    wrapper.getHtmlRichtext().toString()
                } else {
                    wrapper.getHtmlContent().toString()
                }
            }
        }
    }

    fun bindListener(commentWrapper: CommentWrapperInterface,
                             view: View?, viewHolder: RecyclerView.ViewHolder, position: Int) {

        view?.setTag(R.id.commentItemViewHolder, viewHolder)
        with(viewHolder.itemView) {
            applyTagsAndListener(commentWrapper, this, viewHolder, position)
            applyTagsAndListener(commentWrapper, avatar, viewHolder, position)
            applyTagsAndListener(commentWrapper, userName, viewHolder, position)
            applyTagsAndListener(commentWrapper, replyBtnV4, viewHolder, position)
            applyTagsAndListener(commentWrapper, likeBtn, viewHolder, position)
            applyTagsAndListener(commentWrapper, dislikeBtn, viewHolder, position)
            applyTagsAndListener(commentWrapper, moreBtn, viewHolder, position)
            applyTagsAndListener(commentWrapper, loadMoreTxt, viewHolder, position)
            applyTagsAndListener(commentWrapper, proBadge, viewHolder, position)
            applyTagsAndListener(commentWrapper, refresh, viewHolder, position)
        }
    }

    private fun applyTagsAndListener(commentWrapper: CommentWrapperInterface,
                                     view: View?, viewHolder: RecyclerView.ViewHolder, position: Int) {
        view?.apply {
            tag = commentWrapper
            setTag(R.id.commentPosition, position)
            setTag(R.id.commentItemViewHolder, viewHolder)
            setOnClickListener(clickListener)
        }
    }

    fun applyVoteInfo(wrapper: CommentWrapperInterface,
                      holder: RecyclerView.ViewHolder, themeAttr: CommentItemThemeAttr) {
        with(holder.itemView) {
            commentItemActionDelegate.setLikeStatus(context, likeBtn, likeBtn, wrapper, false)
            commentItemActionDelegate.setDislikeStatus(context, dislikeBtn, dislikeBtn, wrapper, false)
            meta.text = wrapper.getTimeText(themeAttr.timeFormatter)
        }
    }

    private fun getLinkMovementMethod(eventScope: String): LinkMovementMethod {
        if (!::linkMovementMethod.isInitialized) {
            linkMovementMethod = BetterLinkMovementMethod.getInstance()
        }
        return linkMovementMethod
    }
}