package com.under9.android.comments.ui.widget

import android.content.Context
import android.util.AttributeSet
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.widget.*
import androidx.annotation.AttrRes
import androidx.annotation.DrawableRes
import androidx.annotation.StyleRes
import androidx.appcompat.content.res.AppCompatResources
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.dynamicanimation.animation.SpringAnimation
import androidx.dynamicanimation.animation.SpringForce
import com.facebook.drawee.view.SimpleDraweeView
import com.under9.android.commentsystem.R
import com.under9.android.lib.util.L10nUtil
import com.under9.android.lib.util.StringUtils
import com.under9.android.lib.util.UiUtil
import com.under9.android.lib.widget.ActiveAvatarView
import com.under9.android.lib.widget.ProBadgeView
import com.under9.android.lib.widget.TextDrawable
import com.under9.android.lib.widget.uiv3.UniversalImageView

class CommentItemView : ConstraintLayout {
    constructor(context: Context): this(context, null, -1)

    constructor(context: Context, attributeSet: AttributeSet?) : this(context, attributeSet, -1)

    constructor(context: Context, @AttrRes attributeSet: AttributeSet?, @StyleRes defStyleAttr: Int) : super(context, attributeSet, defStyleAttr) {
        attributeSet?.let {
            loadAttributes(context, attributeSet, defStyleAttr)
        }
    }

    val root: ConstraintLayout
    val avatar: ActiveAvatarView
    val userName: TextView
    val proBadge: ProBadgeView
    val meta: TextView
    val content: TextView
    val uiv: UniversalImageView
    val replyBtn: TextView
    val likeBtn: CheckBox
    val likeBtnMask: CheckBox
    val dislikeBtn: CheckBox
    val dislikeBtnMask: CheckBox
    val moreBtn: ImageButton
    val loadMoreTxt: TextView
    val progressBar: ProgressBar
    val refresh: ImageView
    val commentBorder: View

    init {
        LayoutInflater.from(context).inflate(R.layout.comment_system_list_item_inner_v5, this, true)
        root = this
        avatar = findViewById(R.id.avatar)
        userName = findViewById(R.id.userName)
        proBadge = findViewById(R.id.proBadge)
        meta = findViewById(R.id.meta)
        content = findViewById(R.id.content)
        uiv = findViewById(R.id.image)
        replyBtn = findViewById(R.id.replyBtnV4)
        likeBtn = findViewById(R.id.likeBtn)
        likeBtnMask = findViewById(R.id.likeBtnMask)
        likeBtnMask.isClickable = false
        likeBtnMask.visibility = View.GONE

        dislikeBtn = findViewById(R.id.dislikeBtn)
        dislikeBtnMask = findViewById(R.id.dislikeBtnMask)
        dislikeBtnMask.isClickable = false
        dislikeBtnMask.visibility = View.GONE

        moreBtn = findViewById(R.id.moreBtn)
        loadMoreTxt = findViewById(R.id.loadMoreTxt)
        progressBar = findViewById(R.id.progress)
        refresh = findViewById(R.id.refresh)
        commentBorder = findViewById(R.id.commentBorder)
    }

    private fun loadAttributes(context: Context, attrs: AttributeSet, deftStyleAttr: Int) {
        val a = context.theme.obtainStyledAttributes(attrs,
                R.styleable.CommentItemView, deftStyleAttr, 0)
        val enableVoteMask = a.getBoolean(R.styleable.CommentItemView_enable_vote_mask, false)
        if (enableVoteMask) {
            likeBtnMask.visibility = View.VISIBLE
            dislikeBtnMask.visibility = View.VISIBLE
        } else {
            likeBtnMask.visibility = View.GONE
            dislikeBtnMask.visibility = View.GONE
        }
    }

    fun setAvatar(uri: String?) {
        this.avatar.setImageURI(uri)
    }

    fun setRoundAvatarColorDrawable(colors: IntArray,
                                    cache: Array<TextDrawable?>,
                                    @DrawableRes imageDrawable: Int,
                                    username: String) {
        val hashedColorIndex = StringUtils.getHashedSlot(username, colors.size)
        if (cache[hashedColorIndex] == null) {
            cache[hashedColorIndex] = TextDrawable.builder()
                    .buildRound("", colors[hashedColorIndex])
        }

        this.avatar.setImageBackground(cache[hashedColorIndex])
        this.avatar.setImageDrawable(AppCompatResources.getDrawable(context, imageDrawable))
    }

    fun setPointStatus(userPoint: Int) {
        var likeSelected = false
        var dislikeSelected = false

        if (userPoint == -1) {
            dislikeSelected = true
        } else if (userPoint == 1) {
            likeSelected = true
        }

        likeBtn.isChecked = likeSelected
        likeBtnMask.isChecked = likeSelected

        dislikeBtn.isChecked = dislikeSelected
        dislikeBtn.isChecked = dislikeSelected
    }

    fun setLikeStatus(likeCount: Int, dislikeCount: Int,
                      status: Int, shouldHighlight: Boolean, withAnimation: Boolean) {
        setVoteStatus(likeBtn, likeCount, status, shouldHighlight)
        setVoteStatus(dislikeBtn, dislikeCount, -status, shouldHighlight)

        likeBtn.isChecked = status == 1
        likeBtnMask.isChecked = likeBtn.isChecked

        dislikeBtn.isChecked = false
        dislikeBtnMask.isChecked = false

        if (withAnimation) {
            runAnimation(likeBtn, status)
        }
    }

    fun setDiskLikeStatus(likeCount: Int, dislikeCount: Int,
                          status: Int, shouldHighlight: Boolean, withAnimation: Boolean) {
        setVoteStatus(likeBtn, likeCount, status, shouldHighlight)
        setVoteStatus(dislikeBtn, dislikeCount, -status, shouldHighlight)

        likeBtn.isChecked = false
        likeBtnMask.isChecked = false

        dislikeBtn.isChecked = status == -1
        dislikeBtnMask.isChecked = dislikeBtn.isChecked

        if (withAnimation) {
            runAnimation(dislikeBtn, status)
        }
    }

    private fun runAnimation(v: View, status: Int) {
        val startValue = when (status) {
            1 -> -10f
            -1 -> 10f
            else -> 0f
        }

        val animation = SpringAnimation(v, SpringAnimation.TRANSLATION_Y)
        val springForce = SpringForce()
        springForce.finalPosition = 0f
        springForce.stiffness = SpringForce.STIFFNESS_MEDIUM
        springForce.dampingRatio = SpringForce.DAMPING_RATIO_MEDIUM_BOUNCY
        animation.setStartValue(startValue)
        animation.spring = springForce
        animation.start()
    }

    private fun setVoteStatus(countTextView: TextView,
                              count: Int, status: Int, shouldHighlight: Boolean) {
        if (!shouldHighlight) {
            val color = getLikeCountColorByStatus(status)
            val resolvedColor = UiUtil.themeAttributeToColor(color, context, -1)
            countTextView.setTextColor(resolvedColor)
        }

        if (count == 0) {
            countTextView.text = ""
        } else {
            countTextView.text = L10nUtil.getFormatNumber(count)
        }
    }

    @AttrRes
    private fun getLikeCountColorByStatus(likeStatus: Int): Int {
        return when {
            likeStatus > 0 -> R.attr.under9_themeColorAccent
            likeStatus == 0 -> R.attr.under9_themeTextColorSecondary
            else -> R.attr.under9_themeTextColorSecondary
        }
    }
}