package com.under9.android.lib.widget

import android.annotation.TargetApi
import android.content.Context
import android.graphics.Color
import android.graphics.PorterDuff
import android.graphics.Typeface
import android.graphics.drawable.Drawable
import android.os.Build
import android.util.AttributeSet
import android.util.TypedValue
import android.view.Gravity
import android.view.ViewGroup
import android.widget.FrameLayout
import android.widget.TextView
import androidx.core.content.ContextCompat
import androidx.core.graphics.drawable.DrawableCompat
import com.under9.android.lib.util.UiUtil


class ProBadgeView : FrameLayout {

    private var isLargeBadge = false
    private var fixedBadgeColor: Int = Int.MAX_VALUE
    private var fixedBadgeTextColor: Int = Int.MAX_VALUE

    private lateinit var proBadgeTextView: TextView

    companion object {
        const val BADGE_SMALL_TEXT_SIZE = 10f
        const val BADGE_LARGE_TEXT_SIZE = 14f
    }

    @JvmOverloads
    constructor(
            context: Context,
            attrs: AttributeSet? = null,
            defStyleAttr: Int = 0)
            : super(context, attrs, defStyleAttr) {
        init(attrs)
    }

    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    constructor(
            context: Context,
            attrs: AttributeSet?,
            defStyleAttr: Int,
            defStyleRes: Int)
            : super(context, attrs, defStyleAttr, defStyleRes) {
        init(attrs)
    }

    private fun init(attrs: AttributeSet?) {
        proBadgeTextView = TextView(context)
        proBadgeTextView.layoutParams = ViewGroup.LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT)
        proBadgeTextView.gravity = Gravity.CENTER
        attrs?.let {
            val typedArray = context.obtainStyledAttributes(it, R.styleable.ProBadgeView, 0, 0)
            isLargeBadge = typedArray.getBoolean(R.styleable.ProBadgeView_isLargeBadge, false)
            fixedBadgeColor = typedArray.getColor(R.styleable.ProBadgeView_fixedBadgeColor, Int.MAX_VALUE)
            fixedBadgeTextColor = typedArray.getColor(R.styleable.ProBadgeView_fixedBadgeTextColor, Int.MAX_VALUE)
            // -1 is represent the white color
            if (fixedBadgeColor == -1) {
                fixedBadgeColor = Color.WHITE
                proBadgeTextView.setTextColor(Color.BLACK)
            }

            typedArray.recycle()
        }
        updateProBadge(false)
    }

    fun updateProBadge(isProPlus: Boolean) {
        val badgeColor = if (fixedBadgeColor != Int.MAX_VALUE) {
            fixedBadgeColor
        } else {
            if (!isProPlus) {
                ContextCompat.getColor(context, R.color.under9_theme_yellow)
            } else {
                ContextCompat.getColor(context, R.color.under9_theme_orange)
            }
        }
        if (isLargeBadge) toLargeBadge(badgeColor) else toSmallBadge(badgeColor)
        if (isProPlus) toProPlus() else toPro()

        if (childCount > 0) {
            removeAllViews()
        }
        addView(proBadgeTextView)
    }

    private fun toPro() {
        with(proBadgeTextView) {
            text = context.getString(R.string.pro_badge_text)
        }
    }

    private fun toProPlus() {
        with(proBadgeTextView) {
            text = context.getString(R.string.pro_plus_badge_text)
        }
    }

    private fun toLargeBadge(badgeColor: Int) {
        applyRoundBgWithColor(badgeColor, ContextCompat.getDrawable(context, R.drawable.pro_badge_large_bg)!!)
        proBadgeTextView.setTextSize(TypedValue.COMPLEX_UNIT_SP, BADGE_LARGE_TEXT_SIZE)
        applyTextViewLayout()
    }

    private fun toSmallBadge(badgeColor: Int) {
        applyRoundBgWithColor(badgeColor, ContextCompat.getDrawable(context, R.drawable.pro_badge_small_bg)!!)
        proBadgeTextView.setTextSize(TypedValue.COMPLEX_UNIT_SP, BADGE_SMALL_TEXT_SIZE)
        applyTextViewLayout()
    }

    private fun applyTextViewLayout() {
        with(proBadgeTextView) {
            gravity = Gravity.CENTER
            typeface = Typeface.DEFAULT_BOLD
            val defaultTextColor = UiUtil.themeAttributeToColor(R.attr.under9_themeTextColorPrimaryInverse, context, -1)
            if (fixedBadgeTextColor == Int.MAX_VALUE || fixedBadgeTextColor == defaultTextColor) {
                setTextColor(defaultTextColor)
            } else {
                setTextColor(fixedBadgeTextColor)
            }
        }
        val leftRight = context.resources.getDimension(R.dimen.pro_badge_padding_left_right).toInt()
        val topBottom = context.resources.getDimension(R.dimen.pro_badge_padding_top_bottom).toInt()
        setPadding(leftRight, topBottom, leftRight, topBottom)
    }

    private fun applyRoundBgWithColor(tintColor: Int, drawable: Drawable) {
        val wrapDrawable = DrawableCompat.wrap(drawable).also {
            DrawableCompat.setTint(it, tintColor)
            DrawableCompat.setTintMode(it, PorterDuff.Mode.SRC_IN)
        }
        background = wrapDrawable
    }
}